@@ -1,7 +1,6 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Lang = imports.lang;
-const DBus = imports.dbus;
 const Gio = imports.gi.Gio;
 const St = imports.gi.St;
 
@@ -62,25 +61,19 @@ function startAppForMount(app, mount) {
 
 /******************************************/
 
-const HotplugSnifferIface = {
-    name: 'org.gnome.Shell.HotplugSniffer',
-    methods: [{ name: 'SniffURI',
-                inSignature: 's',
-                outSignature: 'as' }]
-};
+const HotplugSnifferIface = <interface name="org.gnome.Shell.HotplugSniffer">
+<method name="SniffURI">
+    <arg type="s" direction="in" />
+    <arg type="as" direction="out" />
+</method>
+</interface>;
 
-const HotplugSniffer = function() {
-    this._init();
-};
-
-HotplugSniffer.prototype = {
-    _init: function() {
-        DBus.session.proxifyObject(this,
+const HotplugSnifferProxy = Gio.DBusProxy.makeProxyWrapper(HotplugSnifferIface);
+function HotplugSniffer() {
+    return new HotplugSnifferProxy(Gio.DBus.session,
                                    'org.gnome.Shell.HotplugSniffer',
                                    '/org/gnome/Shell/HotplugSniffer');
-    },
-};
-DBus.proxifyPrototype(HotplugSniffer.prototype, HotplugSnifferIface);
+}
 
 function ContentTypeDiscoverer(callback) {
     this._init(callback);
@@ -114,9 +107,8 @@ ContentTypeDiscoverer.prototype = {
             let root = mount.get_root();
 
             let hotplugSniffer = new HotplugSniffer();
-            hotplugSniffer.SniffURIRemote
-                (root.get_uri(), DBus.CALL_FLAG_START,
-                 Lang.bind(this, function(contentTypes) {
+            hotplugSniffer.SniffURIRemote(root.get_uri(),
+                 Lang.bind(this, function([contentTypes]) {
                      this._emitCallback(mount, contentTypes);
                  }));
         }
@@ -491,7 +483,7 @@ AutorunTransientDispatcher.prototype = {
         let app = null;
 
         if (setting == AutorunSetting.RUN) {
-            app = Gio.app_info_get_default_for_type(type, false);
+            app = Gio.app_info_get_default_for_type(contentTypes[0], false);
         } else if (setting == AutorunSetting.FILES) {
             app = Gio.app_info_get_default_for_type('inode/directory', false);
         }
