@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
@@ -12,6 +12,7 @@ const Signals = imports.signals;
 const FileUtils = imports.misc.fileUtils;
 const Main = imports.ui.main;
 const ModalDialog = imports.ui.modalDialog;
+const ShellEntry = imports.ui.shellEntry;
 const Tweener = imports.ui.tweener;
 const Util = imports.misc.util;
 const History = imports.misc.history;
@@ -210,6 +211,7 @@ __proto__: ModalDialog.ModalDialog.prototype,
         this.contentLayout.add(label, { y_align: St.Align.START });
 
         let entry = new St.Entry({ style_class: 'run-dialog-entry' });
+        ShellEntry.addContextMenu(entry);
 
         this._entryText = entry.clutter_text;
         this.contentLayout.add(entry, { y_align: St.Align.START });
@@ -333,33 +335,46 @@ __proto__: ModalDialog.ModalDialog.prototype,
 
                 if (GLib.file_test(path, GLib.FileTest.EXISTS)) {
                     let file = Gio.file_new_for_path(path);
-                    Gio.app_info_launch_default_for_uri(file.get_uri(),
-                                                        global.create_app_launch_context());
-                } else {
-                    this._commandError = true;
-
-                    this._errorMessage.set_text(e.message);
-
-                    if (!this._errorBox.visible) {
-                        let [errorBoxMinHeight, errorBoxNaturalHeight] = this._errorBox.get_preferred_height(-1);
-
-                        let parentActor = this._errorBox.get_parent();
-                        Tweener.addTween(parentActor,
-                                         { height: parentActor.height + errorBoxNaturalHeight,
-                                           time: DIALOG_GROW_TIME,
-                                           transition: 'easeOutQuad',
-                                           onComplete: Lang.bind(this,
-                                               function() {
-                                                    parentActor.set_height(-1);
-                                                    this._errorBox.show();
-                                               })
-                                         });
+                    try {
+                        Gio.app_info_launch_default_for_uri(file.get_uri(),
+                                                            global.create_app_launch_context());
+                    } catch (e) {
+                        // The exception from gjs contains an error string like:
+                        //     Error invoking Gio.app_info_launch_default_for_uri: No application
+                        //     is registered as handling this file
+                        // We are only interested in the part after the first colon.
+                        let message = e.message.replace(/[^:]*: *(.+)/, '$1');
+                        this._showError(message);
                     }
+                } else {
+                    this._showError(e.message);
                 }
             }
         }
     },
 
+    _showError : function(message) {
+        this._commandError = true;
+
+        this._errorMessage.set_text(message);
+
+        if (!this._errorBox.visible) {
+            let [errorBoxMinHeight, errorBoxNaturalHeight] = this._errorBox.get_preferred_height(-1);
+
+            let parentActor = this._errorBox.get_parent();
+            Tweener.addTween(parentActor,
+                             { height: parentActor.height + errorBoxNaturalHeight,
+                               time: DIALOG_GROW_TIME,
+                               transition: 'easeOutQuad',
+                               onComplete: Lang.bind(this,
+                                                     function() {
+                                                         parentActor.set_height(-1);
+                                                         this._errorBox.show();
+                                                     })
+                             });
+        }
+    },
+
     open: function() {
         this._history.lastItem();
         this._errorBox.hide();
