@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
 const Gdk = imports.gi.Gdk;
@@ -16,9 +16,6 @@ const MessageTray = imports.ui.messageTray;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
 
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
-
 const ConnectionState = {
     DISCONNECTED: 0,
     CONNECTED: 1,
@@ -67,10 +64,9 @@ Indicator.prototype = {
 
         this._fullMenuItems = [new PopupMenu.PopupSeparatorMenuItem(),
                                new PopupMenu.PopupMenuItem(_("Send Files to Device...")),
-                               new PopupMenu.PopupMenuItem(_("Setup a New Device...")),
+                               new PopupMenu.PopupMenuItem(_("Set up a New Device...")),
                                new PopupMenu.PopupSeparatorMenuItem()];
         this._hasDevices = false;
-        this._deviceSep = this._fullMenuItems[0]; // hidden if no device exists
 
         this._fullMenuItems[1].connect('activate', function() {
             GLib.spawn_command_line_async('bluetooth-sendto');
@@ -92,9 +88,7 @@ Indicator.prototype = {
         this._applet.connect('notify::show-full-menu', Lang.bind(this, this._updateFullMenu));
         this._updateFullMenu();
 
-        this.menu.addAction(_("Bluetooth Settings"), function() {
-            GLib.spawn_command_line_async('gnome-control-center bluetooth');
-        });
+        this.menu.addSettingsAction(_("Bluetooth Settings"), 'bluetooth-properties.desktop');
 
         this._applet.connect('pincode-request', Lang.bind(this, this._pinRequest));
         this._applet.connect('confirm-request', Lang.bind(this, this._confirmRequest));
@@ -110,7 +104,11 @@ Indicator.prototype = {
                          current_state != GnomeBluetoothApplet.KillswitchState.HARD_BLOCKED;
 
         this._killswitch.setToggleState(on);
-        this._killswitch.actor.reactive = can_toggle;
+        if (can_toggle)
+            this._killswitch.setStatus(null);
+        else
+            /* TRANSLATORS: this means that bluetooth was disabled by hardware rfkill */
+            this._killswitch.setStatus(_("hardware disabled"));
 
         if (has_adapter)
             this.actor.show();
@@ -126,47 +124,31 @@ Indicator.prototype = {
         }
     },
 
-    _deviceCompare: function(d1, d2) {
-        return d1.device_path == d2.device_path &&
-            d1.bdaddr == d2.bdaddr &&
-            d1.can_connect == d2.can_connect &&
-            d1.capabilities == d2.capabilities;
-    },
-
     _updateDevices: function() {
         let devices = this._applet.get_devices();
 
-        for (let i = 0; i < this._deviceItems.length; i++) {
-            let item = this._deviceItems[i];
-            let destroy = true;
-            for (let j = 0; j < devices.length; j++) {
-                // we need to deep compare because BluetoothSimpleDevice is a boxed type
-                // (but we take advantage of that, because _skip will disappear the next
-                // time get_devices() is called)
-                if (this._deviceCompare(item._device, devices[i])) {
-                    item.label.text = devices[i].alias;
-                    devices[i]._skip = true;
-                    destroy = false;
-                }
-            }
-            if (destroy) {
-                item.destroy();
-                item._destroyed = true;
-            }
-        }
-
         let newlist = [ ];
         for (let i = 0; i < this._deviceItems.length; i++) {
             let item = this._deviceItems[i];
-            if (!item._destroyed)
+            let destroy = true;
+            for (let j = 0; j < devices.length; j++) {
+                if (item._device.device_path == devices[j].device_path) {
+                    this._updateDeviceItem(item, devices[j]);
+                    destroy = false;
+                    break;
+                }
+            }
+            if (destroy)
+                item.destroy();
+            else
                 newlist.push(item);
         }
-        this._deviceItems = newlist;
 
+        this._deviceItems = newlist;
         this._hasDevices = newlist.length > 0;
         for (let i = 0; i < devices.length; i++) {
             let d = devices[i];
-            if (d._skip)
+            if (d._item)
                 continue;
             let item = this._createDeviceItem(d);
             if (item) {
@@ -175,29 +157,65 @@ Indicator.prototype = {
                 this._hasDevices = true;
             }
         }
-        if (this._hasDevices)
-            this._deviceSep.actor.show();
-        else
-            this._deviceSep.actor.hide();
+    },
+
+    _updateDeviceItem: function(item, device) {
+        if (!device.can_connect && device.capabilities == GnomeBluetoothApplet.Capabilities.NONE) {
+            item.destroy();
+            return;
+        }
+
+        let prevDevice = item._device;
+        let prevCapabilities = prevDevice.capabilities;
+        let prevCanConnect = prevDevice.can_connect;
+
+        // adopt the new device object
+        item._device = device;
+        device._item = item;
+
+        // update properties
+        item.label.text = device.alias;
+
+        if (prevCapabilities != device.capabilities ||
+            prevCanConnect != device.can_connect) {
+            // need to rebuild the submenu
+            item.menu.removeAll();
+            this._buildDeviceSubMenu(item, device);
+        }
+
+        // update connected property
+        if (device.can_connect)
+            item._connectedMenuitem.setToggleState(device.connected);
     },
 
     _createDeviceItem: function(device) {
         if (!device.can_connect && device.capabilities == GnomeBluetoothApplet.Capabilities.NONE)
             return null;
         let item = new PopupMenu.PopupSubMenuMenuItem(device.alias);
+
+        // adopt the device object, and add a back link
         item._device = device;
+        device._item = item;
 
+        this._buildDeviceSubMenu(item, device);
+
+        return item;
+    },
+
+    _buildDeviceSubMenu: function(item, device) {
         if (device.can_connect) {
-            item._connected = device.connected;
             let menuitem = new PopupMenu.PopupSwitchMenuItem(_("Connection"), device.connected);
-
+            item._connected = device.connected;
+            item._connectedMenuItem = menuitem;
             menuitem.connect('toggled', Lang.bind(this, function() {
                 if (item._connected > ConnectionState.CONNECTED) {
                     // operation already in progress, revert
+                    // (should not happen anyway)
                     menuitem.setToggleState(menuitem.state);
                 }
                 if (item._connected) {
                     item._connected = ConnectionState.DISCONNECTING;
+                    menuitem.setStatus(_("disconnecting..."));
                     this._applet.disconnect_device(item._device.device_path, function(applet, success) {
                         if (success) { // apply
                             item._connected = ConnectionState.DISCONNECTED;
@@ -206,9 +224,11 @@ Indicator.prototype = {
                             item._connected = ConnectionState.CONNECTED;
                             menuitem.setToggleState(true);
                         }
+                        menuitem.setStatus(null);
                     });
                 } else {
                     item._connected = ConnectionState.CONNECTING;
+                    menuitem.setStatus(_("connecting..."));
                     this._applet.connect_device(item._device.device_path, function(applet, success) {
                         if (success) { // apply
                             item._connected = ConnectionState.CONNECTED;
@@ -217,6 +237,7 @@ Indicator.prototype = {
                             item._connected = ConnectionState.DISCONNECTED;
                             menuitem.setToggleState(false);
                         }
+                        menuitem.setStatus(null);
                     });
                 }
             }));
@@ -248,27 +269,19 @@ Indicator.prototype = {
 
         switch (device.type) {
         case GnomeBluetoothApplet.Type.KEYBOARD:
-            item.menu.addAction(_("Keyboard Settings"), function() {
-                GLib.spawn_command_line_async('gnome-control-center keyboard');
-            });
+            item.menu.addSettingsAction(_("Keyboard Settings"), 'gnome-keyboard-panel.desktop');
             break;
         case GnomeBluetoothApplet.Type.MOUSE:
-            item.menu.addAction(_("Mouse Settings"), function() {
-                GLib.spawn_command_line_async('gnome-control-center mouse');
-            });
+            item.menu.addSettingsAction(_("Mouse Settings"), 'gnome-mouse-panel.desktop');
             break;
         case GnomeBluetoothApplet.Type.HEADSET:
         case GnomeBluetoothApplet.Type.HEADPHONES:
         case GnomeBluetoothApplet.Type.OTHER_AUDIO:
-            item.menu.addAction(_("Sound Settings"), function() {
-                GLib.spawn_command_line_async('gnome-control-center sound');
-            });
+            item.menu.addSettingsAction(_("Sound Settings"), 'gnome-sound-panel.desktop');
             break;
         default:
             break;
         }
-
-        return item;
     },
 
     _updateFullMenu: function() {
@@ -276,8 +289,6 @@ Indicator.prototype = {
             this._showAll(this._fullMenuItems);
             if (this._hasDevices)
                 this._showAll(this._deviceItems);
-            else
-                this._deviceSep.actor.hide();
         } else {
             this._hideAll(this._fullMenuItems);
             this._hideAll(this._deviceItems);
@@ -471,9 +482,15 @@ PinNotification.prototype = {
 
         this.connect('action-invoked', Lang.bind(this, function(self, action) {
             if (action == 'ok') {
-                if (this._numeric)
-                    this._applet.agent_reply_passkey(this._devicePath, parseInt(this._entry.text));
-                else
+                if (this._numeric) {
+                    let num = parseInt(this._entry.text);
+                    if (isNaN(num)) {
+                        // user reply was empty, or was invalid
+                        // cancel the operation
+                        num = -1;
+                    }
+                    this._applet.agent_reply_passkey(this._devicePath, num);
+                } else
                     this._applet.agent_reply_pincode(this._devicePath, this._entry.text);
             } else {
                 if (this._numeric)
