@@ -1,6 +1,5 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
-const DBus = imports.dbus;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Lang = imports.lang;
@@ -900,7 +899,7 @@ ChatNotification.prototype = {
         }
 
         let groups = this._contentArea.get_children();
-        for (let i = 0; i < groups.length; i ++) {
+        for (let i = 0; i < groups.length; i++) {
             let group = groups[i];
             if (group.get_children().length == 0)
                 group.destroy();
@@ -938,7 +937,7 @@ ChatNotification.prototype = {
         let body = highlighter.actor;
 
         let styles = props.styles;
-        for (let i = 0; i < styles.length; i ++)
+        for (let i = 0; i < styles.length; i++)
             body.add_style_class_name(styles[i]);
 
         let group = props.group;
@@ -1297,7 +1296,7 @@ MultiNotificationSource.prototype = {
     },
 
     createNotificationIcon: function() {
-        return new St.Icon({ gicon: Shell.util_icon_from_string(this._icon),
+        return new St.Icon({ gicon: Gio.icon_new_for_string(this._icon),
                              icon_type: St.IconType.FULLCOLOR,
                              icon_size: this.ICON_SIZE });
     }
