@@ -3,6 +3,7 @@
 #include "config.h"
 
 #include "shell-app-system.h"
+#include "shell-app-usage.h"
 #include <string.h>
 
 #include <gio/gio.h>
@@ -670,36 +671,30 @@ shell_app_system_get_running (ShellAppSystem *self)
 
 
 static gint
-compare_apps_by_name (gconstpointer a,
-                      gconstpointer b,
-                      gpointer      data)
+compare_apps_by_usage (gconstpointer a,
+                       gconstpointer b,
+                       gpointer      data)
 {
+  ShellAppUsage *usage = shell_app_usage_get_default ();
+
   ShellApp *app_a = (ShellApp*)a;
   ShellApp *app_b = (ShellApp*)b;
 
-  return shell_app_compare_by_name (app_a, app_b);
+  return shell_app_usage_compare (usage, "", app_a, app_b);
 }
 
 static GSList *
 sort_and_concat_results (ShellAppSystem *system,
-                         GSList         *multiple_prefix_matches,
                          GSList         *prefix_matches,
-                         GSList         *multiple_substring_matches,
                          GSList         *substring_matches)
 {
-  multiple_prefix_matches = g_slist_sort_with_data (multiple_prefix_matches,
-                                                    compare_apps_by_name,
-                                                    system);
   prefix_matches = g_slist_sort_with_data (prefix_matches,
-                                           compare_apps_by_name,
+                                           compare_apps_by_usage,
                                            system);
-  multiple_substring_matches = g_slist_sort_with_data (multiple_substring_matches,
-                                                       compare_apps_by_name,
-                                                       system);
   substring_matches = g_slist_sort_with_data (substring_matches,
-                                              compare_apps_by_name,
+                                              compare_apps_by_usage,
                                               system);
-  return g_slist_concat (multiple_prefix_matches, g_slist_concat (prefix_matches, g_slist_concat (multiple_substring_matches, substring_matches)));
+  return g_slist_concat (prefix_matches, substring_matches);
 }
 
 /**
@@ -726,9 +721,7 @@ search_tree (ShellAppSystem *self,
              GSList         *terms,
              GHashTable     *apps)
 {
-  GSList *multiple_prefix_results = NULL;
   GSList *prefix_results = NULL;
-  GSList *multiple_subtring_results = NULL;
   GSList *substring_results = NULL;
   GSList *normalized_terms;
   GHashTableIter iter;
@@ -743,14 +736,13 @@ search_tree (ShellAppSystem *self,
       ShellApp *app = value;
       (void)id;
       _shell_app_do_match (app, normalized_terms,
-                           &multiple_prefix_results, &prefix_results,
-                           &multiple_subtring_results, &substring_results);
+                           &prefix_results,
+                           &substring_results);
     }
   g_slist_foreach (normalized_terms, (GFunc)g_free, NULL);
   g_slist_free (normalized_terms);
 
-  return sort_and_concat_results (self, multiple_prefix_results, prefix_results,
-                                  multiple_subtring_results, substring_results);
+  return sort_and_concat_results (self, prefix_results, substring_results);
 
 }
 
@@ -789,9 +781,7 @@ shell_app_system_subsearch (ShellAppSystem   *system,
                             GSList           *terms)
 {
   GSList *iter;
-  GSList *multiple_prefix_results = NULL;
   GSList *prefix_results = NULL;
-  GSList *multiple_substring_results = NULL;
   GSList *substring_results = NULL;
   GSList *normalized_terms = normalize_terms (terms);
 
@@ -800,8 +790,8 @@ shell_app_system_subsearch (ShellAppSystem   *system,
       ShellApp *app = iter->data;
       
       _shell_app_do_match (app, normalized_terms,
-                           &multiple_prefix_results, &prefix_results,
-                           &multiple_substring_results, &substring_results);
+                           &prefix_results,
+                           &substring_results);
     }
   g_slist_foreach (normalized_terms, (GFunc)g_free, NULL);
   g_slist_free (normalized_terms);
@@ -809,7 +799,7 @@ shell_app_system_subsearch (ShellAppSystem   *system,
   /* Note that a shorter term might have matched as a prefix, but
      when extended only as a substring, so we have to redo the
      sort rather than reusing the existing ordering */
-  return sort_and_concat_results (system, multiple_prefix_results, prefix_results, multiple_substring_results, substring_results);
+  return sort_and_concat_results (system, prefix_results, substring_results);
 }
 
 /**
