@@ -636,6 +636,7 @@ shell_global_set_cursor (ShellGlobal *global,
           break;
         case SHELL_CURSOR_POINTING_HAND:
           cursor_type = GDK_HAND2;
+          break;
         case SHELL_CURSOR_DND_UNSUPPORTED_TARGET:
           cursor_type = GDK_X_CURSOR;
           break;
@@ -647,7 +648,7 @@ shell_global_set_cursor (ShellGlobal *global,
 
   gdk_window_set_cursor (global->stage_gdk_window, cursor);
 
-  gdk_cursor_unref (cursor);
+  g_object_unref (cursor);
 }
 
 /**
@@ -856,18 +857,18 @@ _shell_global_get_gjs_context (ShellGlobal *global)
  * overview mode or the "looking glass" debug overlay, that block
  * application and normal key shortcuts.
  *
- * Returns value: %TRUE if we succesfully entered the mode. %FALSE if we couldn't
+ * Returns: %TRUE if we succesfully entered the mode. %FALSE if we couldn't
  *  enter the mode. Failure may occur because an application has the pointer
  *  or keyboard grabbed, because Mutter is in a mode itself like moving a
  *  window or alt-Tab window selection, or because shell_global_begin_modal()
  *  was previouly called.
  */
 gboolean
-shell_global_begin_modal (ShellGlobal *global,
-                          guint32      timestamp)
+shell_global_begin_modal (ShellGlobal       *global,
+                          guint32           timestamp,
+                          MetaModalOptions  options)
 {
-  return meta_plugin_begin_modal (global->plugin, global->stage_xwindow,
-                                  None, 0, timestamp);
+  return meta_plugin_begin_modal (global->plugin, global->stage_xwindow, None, options, timestamp);
 }
 
 /**
@@ -1181,12 +1182,13 @@ shell_global_get_memory_info (ShellGlobal        *global,
   JSContext *context;
   gint64 now;
 
-  memset (meminfo, 0, sizeof (meminfo));
 #ifdef HAVE_MALLINFO
   {
     struct mallinfo info = mallinfo ();
     meminfo->glibc_uordblks = info.uordblks;
   }
+#else
+  meminfo->glibc_uordblks = 0;
 #endif
 
   context = gjs_context_get_native_context (global->js_context);
@@ -1280,9 +1282,17 @@ shell_global_get_pointer (ShellGlobal         *global,
                           int                 *y,
                           ClutterModifierType *mods)
 {
+  GdkDeviceManager *gmanager;
+  GdkDevice *gdevice;
+  GdkScreen *gscreen;
   GdkModifierType raw_mods;
 
-  gdk_display_get_pointer (global->gdk_display, NULL, x, y, &raw_mods);
+  gmanager = gdk_display_get_device_manager (global->gdk_display);
+  gdevice = gdk_device_manager_get_client_pointer (gmanager);
+  gdk_device_get_position (gdevice, &gscreen, x, y);
+  gdk_device_get_state (gdevice,
+                        gdk_screen_get_root_window (gscreen),
+                        NULL, &raw_mods);
   *mods = raw_mods & GDK_MODIFIER_MASK;
 }
 
@@ -1299,9 +1309,18 @@ shell_global_sync_pointer (ShellGlobal *global)
 {
   int x, y;
   GdkModifierType mods;
+  GdkDeviceManager *gmanager;
+  GdkDevice *gdevice;
+  GdkScreen *gscreen;
   ClutterMotionEvent event;
 
-  gdk_display_get_pointer (global->gdk_display, NULL, &x, &y, &mods);
+  gmanager = gdk_display_get_device_manager (global->gdk_display);
+  gdevice = gdk_device_manager_get_client_pointer (gmanager);
+
+  gdk_device_get_position (gdevice, &gscreen, &x, &y);
+  gdk_device_get_state (gdevice,
+                        gdk_screen_get_root_window (gscreen),
+                        NULL, &mods);
 
   event.type = CLUTTER_MOTION;
   event.time = shell_global_get_current_time (global);
@@ -1403,7 +1422,7 @@ shell_global_create_app_launch_context (ShellGlobal *global)
 {
   GdkAppLaunchContext *context;
 
-  context = gdk_app_launch_context_new ();
+  context = gdk_display_get_app_launch_context (global->gdk_display);
   gdk_app_launch_context_set_timestamp (context, shell_global_get_current_time (global));
 
   // Make sure that the app is opened on the current workspace even if
@@ -1945,7 +1964,7 @@ shell_global_screenshot_window (ShellGlobal  *global,
  * will enable a login dialog and run in a more confined
  * way. This type is suitable for the display manager.
  *
- * Returns the type of session gnome-shell is providing.
+ * Returns: the type of session gnome-shell is providing.
  */
 ShellSessionType
 shell_global_get_session_type (ShellGlobal  *global)
