@@ -75,8 +75,6 @@ shell_util_get_file_display_name_if_mount (GFile *file)
   return ret;
 }
 
-#define HOME_NAME_SCHEMA        "org.gnome.nautilus.desktop"
-#define HOME_NAME_KEY           "home-icon-name"
 static char *
 shell_util_get_file_display_for_common_files (GFile *file)
 {
@@ -85,26 +83,11 @@ shell_util_get_file_display_for_common_files (GFile *file)
   compare = g_file_new_for_path (g_get_home_dir ());
   if (g_file_equal (file, compare))
     {
-      GSettings *settings;
-      char *name;
-
       g_object_unref (compare);
-
-      settings = g_settings_new (HOME_NAME_SCHEMA);
-      name = g_settings_get_string (settings, HOME_NAME_KEY);
-      g_object_unref (settings);
-
-      if (!(name && name[0]))
-        {
-          g_free (name);
-          return g_strdup (_("Home Folder"));
-        }
-      else
-        {
-          return name;
-        }
+      /* Translators: this is the same string as the one found in
+       * nautilus */
+      return g_strdup (_("Home"));
     }
-  g_object_unref (compare);
 
   compare = g_file_new_for_path ("/");
   if (g_file_equal (file, compare))
@@ -327,7 +310,7 @@ shell_util_get_label_for_uri (const char *text_uri)
  * Look up the icon that should be associated with a given URI.  Handles
  * various special GNOME-internal cases like x-nautilus-search, etc.
  *
- * Return Value: (transfer none): A new #GIcon
+ * Return Value: (transfer full): A new #GIcon
  */
 GIcon *
 shell_util_get_icon_for_uri (const char *text_uri)
@@ -395,22 +378,6 @@ shell_util_get_icon_for_uri (const char *text_uri)
   return g_themed_icon_new ("gtk-file");
 }
 
-/**
- * shell_util_icon_from_string:
- * @string: a stringified #GIcon
- *
- * A static method equivalent to g_icon_new_for_string, workaround
- * for GJS not being able to represent Gio.Icon (which is an interface,
- * not a class).
- *
- * Returns: (transfer full): the icon which is represented by @string
- */
-GIcon *
-shell_util_icon_from_string (const char *string, GError **error)
-{
-  return g_icon_new_for_string (string, error);
-}
-
 static void
 stop_pick (ClutterActor       *actor,
            const ClutterColor *color)
