@@ -180,20 +180,6 @@ st_texture_cache_finalize (GObject *object)
   G_OBJECT_CLASS (st_texture_cache_parent_class)->finalize (object);
 }
 
-typedef struct {
-  StTextureCache *cache;
-  char *uri;
-  char *mimetype;
-  gboolean thumbnail;
-  GIcon *icon;
-  GtkRecentInfo *recent_info;
-  GtkIconInfo *icon_info;
-  gint width;
-  gint height;
-  StIconColors *colors;
-  gpointer user_data;
-} AsyncIconLookupData;
-
 static gboolean
 compute_pixbuf_scale (gint      width,
                       gint      height,
@@ -209,14 +195,15 @@ compute_pixbuf_scale (gint      width,
 
   if (available_width >= 0 && available_height >= 0)
     {
-      // This should keep the aspect ratio of the image intact, because if
-      // available_width < (available_height * width) / height
-      // than
-      // (available_width * height) / width < available_height
-      // So we are guaranteed to either scale the image to have an available_width
-      // for width and height scaled accordingly OR have the available_height
-      // for height and width scaled accordingly, whichever scaling results
-      // in the image that can fit both available dimensions.
+      /* This should keep the aspect ratio of the image intact, because if
+       * available_width < (available_height * width) / height
+       * then
+       * (available_width * height) / width < available_height
+       * So we are guaranteed to either scale the image to have an available_width
+       * for width and height scaled accordingly OR have the available_height
+       * for height and width scaled accordingly, whichever scaling results
+       * in the image that can fit both available dimensions.
+       */
       scaled_width = MIN (available_width, (available_height * width) / height);
       scaled_height = MIN (available_height, (available_width * height) / width);
     }
@@ -235,7 +222,7 @@ compute_pixbuf_scale (gint      width,
       scaled_width = scaled_height = 0;
     }
 
-  // Scale the image only if that will not increase its original dimensions.
+  /* Scale the image only if that will not increase its original dimensions. */
   if (scaled_width > 0 && scaled_height > 0 && scaled_width < width && scaled_height < height)
     {
       *new_width = scaled_width;
@@ -306,32 +293,62 @@ impl_load_pixbuf_gicon (GIcon        *icon,
   return pixbuf;
 }
 
-// A private structure for keeping width and height.
+/* A private structure for keeping width and height. */
 typedef struct {
   int width;
   int height;
 } Dimensions;
 
+/* This struct corresponds to a request for an texture.
+ * It's creasted when something needs a new texture,
+ * and destroyed when the texture data is loaded. */
+typedef struct {
+  StTextureCache *cache;
+  StTextureCachePolicy policy;
+  char *key;
+  char *checksum;
+
+  gboolean thumbnail;
+  gboolean enforced_square;
+
+  guint width;
+  guint height;
+  GSList *textures;
+
+  GIcon *icon;
+  char *mimetype;
+  GtkIconInfo *icon_info;
+  StIconColors *colors;
+  GtkRecentInfo *recent_info;
+  char *uri;
+} AsyncTextureLoadData;
+
 static void
-icon_lookup_data_destroy (gpointer p)
+texture_load_data_destroy (gpointer p)
 {
-  AsyncIconLookupData *data = p;
+  AsyncTextureLoadData *data = p;
 
   if (data->icon)
     {
       g_object_unref (data->icon);
       gtk_icon_info_free (data->icon_info);
+      if (data->colors)
+        st_icon_colors_unref (data->colors);
     }
   else if (data->uri)
     g_free (data->uri);
+  else if (data->recent_info)
+    gtk_recent_info_unref (data->recent_info);
+
+  if (data->key)
+    g_free (data->key);
+  if (data->checksum)
+    g_free (data->checksum);
   if (data->mimetype)
     g_free (data->mimetype);
-  if (data->recent_info)
-    gtk_recent_info_unref (data->recent_info);
-  if (data->colors)
-    st_icon_colors_unref (data->colors);
 
-  g_free (data);
+  if (data->textures)
+    g_slist_free_full (data->textures, (GDestroyNotify) g_object_unref);
 }
 
 /**
@@ -401,10 +418,11 @@ impl_load_pixbuf_data (const guchar   *data,
   rotated_pixbuf = gdk_pixbuf_apply_embedded_orientation (pixbuf);
   width_after_rotation = gdk_pixbuf_get_width (rotated_pixbuf);
 
-  // There is currently no way to tell if the pixbuf will need to be rotated before it is loaded,
-  // so we only check that once it is loaded, and reload it again if it needs to be rotated in order
-  // to use the available width and height correctly.
-  // http://bugzilla.gnome.org/show_bug.cgi?id=579003
+  /* There is currently no way to tell if the pixbuf will need to be rotated before it is loaded,
+   * so we only check that once it is loaded, and reload it again if it needs to be rotated in order
+   * to use the available width and height correctly.
+   * See http://bugzilla.gnome.org/show_bug.cgi?id=579003
+   */
   if (width_before_rotation != width_after_rotation)
     {
       g_object_unref (pixbuf_loader);
@@ -413,7 +431,7 @@ impl_load_pixbuf_data (const guchar   *data,
 
       pixbuf_loader = gdk_pixbuf_loader_new ();
 
-      // We know that the image will later be rotated, so we reverse the available dimensions.
+      /* We know that the image will later be rotated, so we reverse the available dimensions. */
       available_dimensions.width = available_height;
       available_dimensions.height = available_width;
       g_signal_connect (pixbuf_loader, "size-prepared",
@@ -568,7 +586,7 @@ impl_load_thumbnail (StTextureCache    *cache,
       pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (thumbnail_factory, uri, mime_type);
       if (pixbuf)
         {
-          // we need to save the thumbnail so that we don't need to generate it again in the future
+          /* we need to save the thumbnail so that we don't need to generate it again in the future */
           gnome_desktop_thumbnail_factory_save_thumbnail (thumbnail_factory, pixbuf, uri, mtime);
         }
       else
@@ -601,10 +619,10 @@ load_pixbuf_thread (GSimpleAsyncResult *result,
                     GCancellable *cancellable)
 {
   GdkPixbuf *pixbuf;
-  AsyncIconLookupData *data;
+  AsyncTextureLoadData *data;
   GError *error = NULL;
 
-  data = g_object_get_data (G_OBJECT (result), "load_pixbuf_async");
+  data = g_async_result_get_user_data (G_ASYNC_RESULT (result));
   g_assert (data != NULL);
 
   if (data->thumbnail)
@@ -642,127 +660,6 @@ load_pixbuf_thread (GSimpleAsyncResult *result,
                                                g_object_unref);
 }
 
-/**
- * load_icon_pixbuf_async:
- *
- * Asynchronously load the #GdkPixbuf associated with a #GIcon.  Currently
- * the #GtkIconInfo must have already been provided.
- */
-static void
-load_icon_pixbuf_async (StTextureCache       *cache,
-                        GIcon                *icon,
-                        GtkIconInfo          *icon_info,
-                        gint                  size,
-                        StIconColors         *colors,
-                        GCancellable         *cancellable,
-                        GAsyncReadyCallback   callback,
-                        gpointer              user_data)
-{
-  GSimpleAsyncResult *result;
-  AsyncIconLookupData *data;
-
-  data = g_new0 (AsyncIconLookupData, 1);
-  data->cache = cache;
-  data->icon = g_object_ref (icon);
-  data->icon_info = gtk_icon_info_copy (icon_info);
-  data->width = data->height = size;
-  if (colors)
-    data->colors = st_icon_colors_ref (colors);
-  else
-    data->colors = NULL;
-  data->user_data = user_data;
-
-  result = g_simple_async_result_new (G_OBJECT (cache), callback, user_data, load_icon_pixbuf_async);
-
-  g_object_set_data_full (G_OBJECT (result), "load_pixbuf_async", data, icon_lookup_data_destroy);
-  g_simple_async_result_run_in_thread (result, load_pixbuf_thread, G_PRIORITY_DEFAULT, cancellable);
-
-  g_object_unref (result);
-}
-
-static void
-load_uri_pixbuf_async (StTextureCache     *cache,
-                       const char         *uri,
-                       guint               width,
-                       guint               height,
-                       GCancellable       *cancellable,
-                       GAsyncReadyCallback callback,
-                       gpointer            user_data)
-{
-  GSimpleAsyncResult *result;
-  AsyncIconLookupData *data;
-
-  data = g_new0 (AsyncIconLookupData, 1);
-  data->cache = cache;
-  data->uri = g_strdup (uri);
-  data->width = width;
-  data->height = height;
-  data->user_data = user_data;
-
-  result = g_simple_async_result_new (G_OBJECT (cache), callback, user_data, load_uri_pixbuf_async);
-
-  g_object_set_data_full (G_OBJECT (result), "load_pixbuf_async", data, icon_lookup_data_destroy);
-  g_simple_async_result_run_in_thread (result, load_pixbuf_thread, G_PRIORITY_DEFAULT, cancellable);
-
-  g_object_unref (result);
-}
-
-static void
-load_thumbnail_async (StTextureCache     *cache,
-                      const char         *uri,
-                      const char         *mimetype,
-                      guint               size,
-                      GCancellable       *cancellable,
-                      GAsyncReadyCallback callback,
-                      gpointer            user_data)
-{
-  GSimpleAsyncResult *result;
-  AsyncIconLookupData *data;
-
-  data = g_new0 (AsyncIconLookupData, 1);
-  data->cache = cache;
-  data->uri = g_strdup (uri);
-  data->mimetype = g_strdup (mimetype);
-  data->thumbnail = TRUE;
-  data->width = size;
-  data->height = size;
-  data->user_data = user_data;
-
-  result = g_simple_async_result_new (G_OBJECT (cache), callback, user_data, load_thumbnail_async);
-
-  g_object_set_data_full (G_OBJECT (result), "load_pixbuf_async", data, icon_lookup_data_destroy);
-  g_simple_async_result_run_in_thread (result, load_pixbuf_thread, G_PRIORITY_DEFAULT, cancellable);
-
-  g_object_unref (result);
-}
-
-static void
-load_recent_thumbnail_async (StTextureCache     *cache,
-                             GtkRecentInfo      *info,
-                             guint               size,
-                             GCancellable       *cancellable,
-                             GAsyncReadyCallback callback,
-                             gpointer            user_data)
-{
-  GSimpleAsyncResult *result;
-  AsyncIconLookupData *data;
-
-  data = g_new0 (AsyncIconLookupData, 1);
-  data->cache = cache;
-  data->thumbnail = TRUE;
-  data->recent_info = gtk_recent_info_ref (info);
-  data->width = size;
-  data->height = size;
-  data->user_data = user_data;
-
-  result = g_simple_async_result_new (G_OBJECT (cache), callback, user_data, load_recent_thumbnail_async);
-
-  g_object_set_data_full (G_OBJECT (result), "load_pixbuf_async", data, icon_lookup_data_destroy);
-  g_simple_async_result_run_in_thread (result, load_pixbuf_thread, G_PRIORITY_DEFAULT, cancellable);
-
-  g_object_unref (result);
-}
-
 static GdkPixbuf *
 load_pixbuf_async_finish (StTextureCache *cache, GAsyncResult *result, GError **error)
 {
@@ -772,22 +669,6 @@ load_pixbuf_async_finish (StTextureCache *cache, GAsyncResult *result, GError **
   return g_simple_async_result_get_op_res_gpointer (simple);
 }
 
-typedef struct {
-  StTextureCachePolicy policy;
-  char *key;
-  char *uri;
-  gboolean thumbnail;
-  gboolean enforced_square;
-  char *mimetype;
-  GtkRecentInfo *recent_info;
-  char *checksum;
-  GIcon *icon;
-  GtkIconInfo *icon_info;
-  guint width;
-  guint height;
-  GSList *textures;
-} AsyncTextureLoadData;
-
 static CoglHandle
 pixbuf_to_cogl_handle (GdkPixbuf *pixbuf,
                        gboolean   add_padding)
@@ -941,31 +822,21 @@ on_pixbuf_loaded (GObject      *source,
 out:
   if (texdata)
     cogl_handle_unref (texdata);
-  g_free (data->key);
 
-  if (data->icon)
-    {
-      gtk_icon_info_free (data->icon_info);
-      g_object_unref (data->icon);
-    }
-  else if (data->uri)
-    g_free (data->uri);
-
-  if (data->recent_info)
-    gtk_recent_info_unref (data->recent_info);
-  if (data->mimetype)
-    g_free (data->mimetype);
-
-  /* Alternatively we could weakref and just do nothing if the texture
-     is destroyed */
-  for (iter = data->textures; iter; iter = iter->next)
-    {
-      ClutterTexture *texture = iter->data;
-      g_object_unref (texture);
-    }
+  texture_load_data_destroy (data);
+  g_free (data);
 
   g_clear_error (&error);
-  g_free (data);
+}
+
+static void
+load_texture_async (StTextureCache       *cache,
+                    AsyncTextureLoadData *data)
+{
+  GSimpleAsyncResult *result;
+  result = g_simple_async_result_new (G_OBJECT (cache), on_pixbuf_loaded, data, load_texture_async);
+  g_simple_async_result_run_in_thread (result, load_pixbuf_thread, G_PRIORITY_DEFAULT, NULL);
+  g_object_unref (result);
 }
 
 typedef struct {
@@ -1182,6 +1053,18 @@ load_gicon_with_colors (StTextureCache    *cache,
   GtkIconInfo *info;
   StTextureCachePolicy policy;
 
+  /* Do theme lookups in the main thread to avoid thread-unsafety */
+  theme = cache->priv->icon_theme;
+
+  info = gtk_icon_theme_lookup_by_gicon (theme, icon, size, GTK_ICON_LOOKUP_USE_BUILTIN);
+  if (info == NULL)
+    {
+      /* gah, the icon doesn't exist. Return a blank texture that will never load */
+      texture = CLUTTER_ACTOR (create_default_texture (cache));
+      clutter_actor_set_size (texture, size, size);
+      return texture;
+    }
+
   gicon_string = g_icon_to_string (icon);
   /* A return value of NULL indicates that the icon can not be serialized,
    * so don't have a unique identifier for it as a cache key, and thus can't
@@ -1208,38 +1091,24 @@ load_gicon_with_colors (StTextureCache    *cache,
 
   if (create_texture_and_ensure_request (cache, key, size, policy, &request, &texture))
     {
+      /* If there's an outstanding request, we've just added ourselves to it */
       g_free (key);
-      return texture;
     }
-
-  /* Do theme lookups in the main thread to avoid thread-unsafety */
-  theme = cache->priv->icon_theme;
-
-  info = gtk_icon_theme_lookup_by_gicon (theme, icon, size, GTK_ICON_LOOKUP_USE_BUILTIN);
-  if (info != NULL)
+  else
     {
+      /* Else, make a new request */
+
+      request->cache = cache;
       /* Transfer ownership of key */
       request->key = key;
       request->policy = policy;
       request->icon = g_object_ref (icon);
+      request->colors = colors ? st_icon_colors_ref (colors) : NULL;
       request->icon_info = info;
       request->width = request->height = size;
       request->enforced_square = TRUE;
 
-      load_icon_pixbuf_async (cache, icon, info, size, colors, NULL, on_pixbuf_loaded, request);
-    }
-  else
-    {
-      /* Blah; we failed to find the icon, but we've added our texture to the outstanding
-       * requests.  In that case, just undo what create_texture_and_ensure_request() did.
-       */
-       g_slist_foreach (request->textures, (GFunc) g_object_unref, NULL);
-       g_slist_free (request->textures);
-       g_free (request);
-       g_hash_table_remove (cache->priv->outstanding_requests, key);
-       g_free (key);
-       g_object_unref (texture);
-       texture = NULL;
+      load_texture_async (cache, request);
     }
 
   return CLUTTER_ACTOR (texture);
@@ -1271,12 +1140,6 @@ st_texture_cache_load_gicon (StTextureCache    *cache,
   return load_gicon_with_colors (cache, icon, size, theme_node ? st_theme_node_get_icon_colors (theme_node) : NULL);
 }
 
-typedef struct {
-  gchar *path;
-  gint   grid_width, grid_height;
-  ClutterGroup *group;
-} AsyncImageData;
-
 static ClutterActor *
 load_from_pixbuf (GdkPixbuf *pixbuf)
 {
@@ -1297,6 +1160,21 @@ load_from_pixbuf (GdkPixbuf *pixbuf)
   return CLUTTER_ACTOR (texture);
 }
 
+typedef struct {
+  gchar *path;
+  gint   grid_width, grid_height;
+  ClutterGroup *group;
+} AsyncImageData;
+
+static void
+on_data_destroy (gpointer data)
+{
+  AsyncImageData *d = (AsyncImageData *)data;
+  g_free (d->path);
+  g_object_unref (d->group);
+  g_free (d);
+}
+
 static void
 on_sliced_image_loaded (GObject *source_object,
                         GAsyncResult *res,
@@ -1317,15 +1195,6 @@ on_sliced_image_loaded (GObject *source_object,
     }
 }
 
-static void
-on_data_destroy (gpointer data)
-{
-  AsyncImageData *d = (AsyncImageData *)data;
-  g_free (d->path);
-  g_object_unref (d->group);
-  g_free (d);
-}
-
 static void
 free_glist_unref_gobjects (gpointer p)
 {
@@ -1384,7 +1253,7 @@ load_sliced_image (GSimpleAsyncResult *result,
  * note that the dimensions of the image loaded from @path 
  * should be a multiple of the specified grid dimensions.
  *
- * Returns: (transfer none): A new ClutterGroup
+ * Returns: (transfer none): A new #ClutterGroup
  */
 ClutterGroup *
 st_texture_cache_load_sliced_image (StTextureCache    *cache,
@@ -1481,7 +1350,7 @@ symbolic_names_for_icon (const char *name)
  * @theme_node: (allow-none): a #StThemeNode
  * @name: Name of a themed icon
  * @icon_type: the type of icon to load
- * @size: Size of themed
+ * @size: Size of themed icon
  *
  * Load a themed icon into a texture. See the #StIconType documentation
  * for an explanation of how @icon_type affects the returned icon. The
@@ -1583,13 +1452,14 @@ st_texture_cache_load_uri_async (StTextureCache *cache,
   texture = create_default_texture (cache);
 
   data = g_new0 (AsyncTextureLoadData, 1);
+  data->cache = cache;
   data->key = g_strconcat (CACHE_PREFIX_URI, uri, NULL);
   data->policy = ST_TEXTURE_CACHE_POLICY_NONE;
   data->uri = g_strdup (uri);
   data->width = available_width;
   data->height = available_height;
   data->textures = g_slist_prepend (data->textures, g_object_ref (texture));
-  load_uri_pixbuf_async (cache, uri, available_width, available_height, NULL, on_pixbuf_loaded, data);
+  load_texture_async (cache, data);
 
   return CLUTTER_ACTOR (texture);
 }
@@ -1759,7 +1629,7 @@ st_texture_cache_load_file_to_cogl_texture (StTextureCache *cache,
  * into a cairo surface.  On error, a warning is emitted
  * and %NULL is returned.
  *
- * Returns: (transfer full): a new #cairo_surface_t *
+ * Returns: (transfer full): a new #cairo_surface_t
  */
 cairo_surface_t *
 st_texture_cache_load_file_to_cairo_surface (StTextureCache *cache,
@@ -1984,6 +1854,7 @@ st_texture_cache_load_thumbnail (StTextureCache    *cache,
   if (!texdata)
     {
       data = g_new0 (AsyncTextureLoadData, 1);
+      data->cache = cache;
       data->key = g_strdup (key);
       data->policy = ST_TEXTURE_CACHE_POLICY_FOREVER;
       data->uri = g_strdup (uri);
@@ -1993,7 +1864,7 @@ st_texture_cache_load_thumbnail (StTextureCache    *cache,
       data->height = size;
       data->enforced_square = TRUE;
       data->textures = g_slist_prepend (data->textures, g_object_ref (texture));
-      load_thumbnail_async (cache, uri, mimetype, size, NULL, on_pixbuf_loaded, data);
+      load_texture_async (cache, data);
     }
   else
     {
@@ -2062,6 +1933,7 @@ st_texture_cache_load_recent_thumbnail (StTextureCache    *cache,
   if (!texdata)
     {
       data = g_new0 (AsyncTextureLoadData, 1);
+      data->cache = cache;
       data->key = g_strdup (key);
       data->policy = ST_TEXTURE_CACHE_POLICY_FOREVER;
       data->thumbnail = TRUE;
@@ -2070,7 +1942,7 @@ st_texture_cache_load_recent_thumbnail (StTextureCache    *cache,
       data->height = size;
       data->enforced_square = TRUE;
       data->textures = g_slist_prepend (data->textures, g_object_ref (texture));
-      load_recent_thumbnail_async (cache, info, size, NULL, on_pixbuf_loaded, data);
+      load_texture_async (cache, data);
     }
   else
     {
